function gp = gr_ipa_opt( varargin )
%GR_PLANE_OPT( x, y, op, x_start, y_start )  Constructor

%global color constants
gp.bcgd_color = 'k';
gp.grad_color = 'g';
gp.con_color = 'w';
gp.obj_colormap = 'gray';
gp.path_color = 'm';
gp.clab_color = 'w';
gp.n_of_levels = 15;
gp.min_penalty = 0.001;
gp.max_penalty = 100;
gp.levels = [];

switch nargin
 case 0
  gp.xmin = NaN;
  gp.xmax = NaN;
  gp.x = [];
  
  gp.ymin = NaN;
  gp.ymax = NaN;
  gp.y = [];
  
  gp.x_start = NaN;
  gp.y_start = NaN;
  gp.obj_val = NaN;
  gp.penalty = NaN;
  gp.mu = NaN;
  gp.n_eval = 0;
  
  gp.path = zeros( 0, 3 );
    
  gp.op = opt_prob;
    
  gp = class( gp, 'gr_ipa_opt' );
  
 case 1
  if( isa( varargin{1}, 'gr_ipa_opt' ) )
    p = varargin{1};
  else
    error( 'Wrong argument type' );
  end
 
 case 5
  gp.xmin = min( varargin{1} );
  gp.xmax = max( varargin{1} );
  gp.x = varargin{1};
  
  gp.ymin = min( varargin{2} );
  gp.ymax = max( varargin{2} );
  gp.y = varargin{2};
  
  gp.x_start = varargin{4};
  gp.y_start = varargin{5};
  gp.obj_val = NaN;
  gp.penalty = NaN;
  gp.mu = NaN;
  gp.n_eval = 0;
  
  gp.path = zeros( 0, 3 );
  
  if( isa( varargin{3}, 'ipa_opt_prob' ) )
    gp.op = varargin{3};
  else
    error( 'Wrong argument type' );
  end  
  
  gp = class( gp, 'gr_ipa_opt' );
    
  sx = length( gp.x );
  sy = length( gp.y );
  
  %graphics stuff
  [X Y] = meshgrid( gp.x, gp.y );
  ograd = -obj_grad( gp.op, [reshape(X, 1, sx*sy); ...
		    reshape(Y, 1, sx*sy)] );
  hold( 'off' );
  hs = quiver( X, Y, ...
	  reshape( ograd(1,:), sy, sx), ...
	  reshape( ograd(2,:), sy, sx), ...
	  1, ...
	  gp.grad_color );
  set( hs, ...
	  'HitTest', 'off', ...
          'HandleVisibility', 'off' );

  hold( 'on' );
  [c, hd]  = contour( X, Y, ...
		      reshape( obj_func( gp.op, [reshape(X, 1, sx*sy); ...
		    reshape( Y, 1, sx*sy ) ] ), sy, sx ), ...
		      gp.n_of_levels );
  gp.levels = zeros( length( hd ) );
  for i = 1:length( hd )
    gp.levels(i) = get( hd(i), 'UserData' );
  end
  hs = clabel( c, hd, 'color', gp.clab_color );  
  %  colormap( gray );
  %  shading( 'interp' );
  set( hd, ...
	  'HitTest', 'off', ...
          'HandleVisibility', 'off' );
  set( hs, ...
	  'HitTest', 'off', ...
          'HandleVisibility', 'off' );
  set( gca, 'Color', gp.bcgd_color );
  axis( [gp.xmin, gp.xmax, gp.ymin, gp.ymax] );
  
  XY = [reshape(X, 1, sx*sy); reshape(Y, 1, sx*sy)];
  clear( 'X' ); clear( 'Y' );
  clear( 'ograd' );
  
  for i = 1:n_con( gp.op )
    plot_con( gp, i, XY, sx, sy );
  end
  
  
 otherwise
  error( 'Wrong number of arguments' );
end


function plot_con( gp, i, XY, sx, sy )
c = reshape( con_func( gp.op, i,XY ), sy, sx );
c0 = contourc( gp.x, gp.y, c, [0 0] );

start_ind = 1;
end_ind = length(c0);

hold( 'on' );
while ( start_ind <= end_ind )
  cur_length = c0( 2, start_ind );
  
  plot( c0(1, start_ind+1:start_ind+cur_length), ...
	c0(2, start_ind+1:start_ind+cur_length), ...
	gp.con_color, 'LineWidth', 2, ...
	'HitTest', 'off', ...
	'HandleVisibility', 'off' );
  start_ind = start_ind + cur_length + 1;
end
hold( 'off' );